@@ -1,21 +1,21 @@
 #' HTTP and WebSocket server
-#' 
-#' Allows R code to listen for and interact with HTTP and WebSocket clients, so 
+#'
+#' Allows R code to listen for and interact with HTTP and WebSocket clients, so
 #' you can serve web traffic directly out of your R process. Implementation is
 #' based on \href{https://github.com/joyent/libuv}{libuv} and
 #' \href{https://github.com/joyent/http-parser}{http-parser}.
-#' 
-#' This is a low-level library that provides little more than network I/O and 
-#' implementations of the HTTP and WebSocket protocols. For an easy way to 
+#'
+#' This is a low-level library that provides little more than network I/O and
+#' implementations of the HTTP and WebSocket protocols. For an easy way to
 #' create web applications, try \href{http://rstudio.com/shiny/}{Shiny} instead.
-#' 
+#'
 #' @examples
 #' \dontrun{
 #' demo("echo", package="httpuv")
 #' }
-#' 
+#'
 #' @seealso \link{startServer}
-#'   
+#'
 #' @name httpuv-package
 #' @aliases httpuv
 #' @docType package
@@ -23,45 +23,45 @@
 #' @author Joe Cheng \email{joe@@rstudio.com}
 #' @keywords package
 #' @useDynLib httpuv
-#' @import methods
 #' @importFrom Rcpp evalCpp
 NULL
 
 # Implementation of Rook input stream
-InputStream <- setRefClass(
+InputStream <- R6Class(
   'InputStream',
-  fields = list(
-    .conn = 'ANY',
-    .length = 'numeric'
-  ),
-  methods = list(
+  public = list(
     initialize = function(conn, length) {
-      .conn <<- conn
-      .length <<- length
-      seek(.conn, 0)
+      private$conn <- conn
+      private$length <- length
+      seek(private$conn, 0)
     },
     read_lines = function(n = -1L) {
-      readLines(.conn, n, warn=FALSE)
+      readLines(private$conn, n, warn = FALSE)
     },
     read = function(l = -1L) {
       # l < 0 means read all remaining bytes
       if (l < 0)
-        l <- .length - seek(.conn)
-      
+        l <- private$length - seek(private$conn)
+
       if (l == 0)
         return(raw())
       else
-        return(readBin(.conn, raw(), l))
+        return(readBin(private$conn, raw(), l))
     },
     rewind = function() {
-      seek(.conn, 0)
+      seek(private$conn, 0)
     }
-  )
+  ),
+  private = list(
+    conn = NULL,
+    length = NULL
+  ),
+  cloneable = FALSE
 )
 
-NullInputStream <- setRefClass(
+NullInputStream <- R6Class(
   'NullInputStream',
-  methods = list(
+  public = list(
     read_lines = function(n = -1L) {
       character()
     },
@@ -70,21 +70,23 @@ NullInputStream <- setRefClass(
     },
     rewind = function() invisible(),
     close = function() invisible()
-  )
+  ),
+  cloneable = FALSE
 )
 nullInputStream <- NullInputStream$new()
 
 #Implementation of Rook error stream
-ErrorStream <- setRefClass(
+ErrorStream <- R6Class(
   'ErrorStream',
-  methods = list(
+  public = list(
     cat = function(... , sep = " ", fill = FALSE, labels = NULL) {
       base::cat(..., sep=sep, fill=fill, labels=labels, file=stderr())
     },
     flush = function() {
       base::flush(stderr())
     }
-  )
+  ),
+  cloneable = FALSE
 )
 
 #' @importFrom promises promise then finally is.promise %...>% %...!%
@@ -165,28 +167,59 @@ rookCall <- function(func, req, data = NULL, dataLength = -1) {
   }
 }
 
-AppWrapper <- setRefClass(
+AppWrapper <- R6Class(
   'AppWrapper',
-  fields = list(
-    .app = 'ANY',
-    .wsconns = 'environment',
-    .supportsOnHeaders = 'logical'
+  private = list(
+    app = NULL,                    # List defining app
+    wsconns = NULL,                # An environment containing websocket connections
+    supportsOnHeaders = NULL       # Logical
   ),
-  methods = list(
+  public = list(
     initialize = function(app) {
       if (is.function(app))
-        .app <<- list(call=app)
+        private$app <- list(call=app)
       else
-        .app <<- app
+        private$app <- app
 
-      # .app$onHeaders can error (e.g. if .app is a reference class)
-      .supportsOnHeaders <<- isTRUE(try(!is.null(.app$onHeaders), silent=TRUE))
+      # private$app$onHeaders can error (e.g. if private$app is a reference class)
+      private$supportsOnHeaders <- isTRUE(try(!is.null(private$app$onHeaders), silent=TRUE))
+
+      # staticPaths are saved in a field on this object, because they are read
+      # from the app object only during initialization. This is the only time
+      # it makes sense to read them from the app object, since they're
+      # subsequently used on the background thread, and for performance
+      # reasons it can't call back into R. Note that if the app object is a
+      # reference object and app$staticPaths is changed later, it will have no
+      # effect on the behavior of the application.
+      #
+      # If private$app is a reference class, accessing private$app$staticPaths
+      # can error if not present.
+      if (class(try(private$app$staticPaths, silent = TRUE)) == "try-error" ||
+          is.null(private$app$staticPaths))
+      {
+        self$staticPaths <- list()
+      } else {
+        self$staticPaths <- normalizeStaticPaths(private$app$staticPaths)
+      }
+
+      if (class(try(private$app$staticPathOptions, silent = TRUE)) == "try-error" ||
+          is.null(private$app$staticPathOptions))
+      {
+        # Use defaults
+        self$staticPathOptions <- staticPathOptions()
+      } else if (inherits(private$app$staticPathOptions, "staticPathOptions")) {
+        self$staticPathOptions <- normalizeStaticPathOptions(private$app$staticPathOptions)
+      } else {
+        stop("staticPathOptions must be an object of class staticPathOptions.")
+      }
+
+      private$wsconns <- new.env(parent = emptyenv())
     },
     onHeaders = function(req) {
-      if (!.supportsOnHeaders)
+      if (!private$supportsOnHeaders)
         return(NULL)
 
-      rookCall(.app$onHeaders, req)
+      rookCall(private$app$onHeaders, req)
     },
     onBodyData = function(req, bytes) {
       if (is.null(req$.bodyData))
@@ -197,7 +230,7 @@ AppWrapper <- setRefClass(
       # The cpp_callback is an external pointer to a C++ function that writes
       # the response.
 
-      resp <- rookCall(.app$call, req, req$.bodyData, seek(req$.bodyData))
+      resp <- rookCall(private$app$call, req, req$.bodyData, seek(req$.bodyData))
       # Note: rookCall() should never throw error because all the work is
       # wrapped in tryCatch().
 
@@ -223,8 +256,8 @@ AppWrapper <- setRefClass(
     },
     onWSOpen = function(handle, req) {
       ws <- WebSocket$new(handle, req)
-      .wsconns[[wsconn_address(handle)]] <<- ws
-      result <- try(.app$onWSOpen(ws))
+      private$wsconns[[wsconn_address(handle)]] <- ws
+      result <- try(private$app$onWSOpen(ws))
 
       # If an unexpected error happened, just close up
       if (inherits(result, 'try-error')) {
@@ -232,35 +265,48 @@ AppWrapper <- setRefClass(
       }
     },
     onWSMessage = function(handle, binary, message) {
-      for (handler in .wsconns[[wsconn_address(handle)]]$.messageCallbacks) {
+      for (handler in private$wsconns[[wsconn_address(handle)]]$messageCallbacks) {
         result <- try(handler(binary, message))
         if (inherits(result, 'try-error')) {
-          .wsconns[[wsconn_address(handle)]]$close(1011, "Error executing onWSMessage")
+          private$wsconns[[wsconn_address(handle)]]$close(1011, "Error executing onWSMessage")
           return()
         }
       }
     },
     onWSClose = function(handle) {
-      ws <- .wsconns[[wsconn_address(handle)]]
-      ws$.handle <- NULL
-      .wsconns[[wsconn_address(handle)]] <<- NULL
+      ws <- private$wsconns[[wsconn_address(handle)]]
+      ws$handle <- NULL
+      private$wsconns[[wsconn_address(handle)]] <- NULL
 
-      for (handler in ws$.closeCallbacks) {
+      for (handler in ws$closeCallbacks) {
         handler()
       }
-    }
+    },
+
+    staticPaths = NULL,            # List of static paths
+    staticPathOptions = NULL       # StaticPathOptions object
   )
 )
 
-#' WebSocket object
-#' 
-#' An object that represents a single WebSocket connection. The object can be
-#' used to send messages and close the connection, and to receive notifications
-#' when messages are received or the connection is closed.
-#' 
+#' WebSocket class
+#'
+#' A \code{WebSocket} object represents a single WebSocket connection. The
+#' object can be used to send messages and close the connection, and to receive
+#' notifications when messages are received or the connection is closed.
+#'
+#' Note that this WebSocket class is different from the one provided by the
+#' package named websocket. This class is meant to be used on the server side,
+#' whereas the one in the websocket package is to be used as a client. The
+#' WebSocket class in httpuv has an older API than the one in the websocket
+#' package.
+#'
 #' WebSocket objects should never be created directly. They are obtained by
 #' passing an \code{onWSOpen} function to \code{\link{startServer}}.
-#' 
+#'
+#' @usage NULL
+#'
+#' @format NULL
+#'
 #' @section Fields:
 #'
 #'   \describe{
@@ -270,9 +316,9 @@ AppWrapper <- setRefClass(
 #'     }
 #'   }
 #'
-#' 
+#'
 #' @section Methods:
-#' 
+#'
 #'   \describe{
 #'     \item{\code{onMessage(func)}}{
 #'       Registers a callback function that will be invoked whenever a message
@@ -295,41 +341,33 @@ AppWrapper <- setRefClass(
 #'     }
 #'   }
 #'
-#' @param ... For internal use only.
-#' 
 #' @export
-WebSocket <- setRefClass(
+WebSocket <- R6Class(
   'WebSocket',
-  fields = list(
-    '.handle' = 'ANY',
-    '.messageCallbacks' = 'list',
-    '.closeCallbacks' = 'list',
-    'request' = 'environment'
-  ),
-  methods = list(
+  public = list(
     initialize = function(handle, req) {
-      .handle <<- handle
-      request <<- req
+      self$handle <- handle
+      self$request <- req
     },
     onMessage = function(func) {
-      .messageCallbacks <<- c(.messageCallbacks, func)
+      self$messageCallbacks <- c(self$messageCallbacks, func)
     },
     onClose = function(func) {
-      .closeCallbacks <<- c(.closeCallbacks, func)
+      self$closeCallbacks <- c(self$closeCallbacks, func)
     },
     send = function(message) {
-      if (is.null(.handle))
+      if (is.null(self$handle))
         return()
-      
+
       if (is.raw(message))
-        sendWSMessage(.handle, TRUE, message)
+        sendWSMessage(self$handle, TRUE, message)
       else {
         # TODO: Ensure that message is UTF-8 encoded
-        sendWSMessage(.handle, FALSE, as.character(message))
+        sendWSMessage(self$handle, FALSE, as.character(message))
       }
     },
     close = function(code = 1000L, reason = "") {
-      if (is.null(.handle))
+      if (is.null(self$handle))
         return()
 
       # Make sure the code will fit in a short int (2 bytes); if not just use
@@ -341,26 +379,31 @@ WebSocket <- setRefClass(
       }
       reason <- iconv(reason, to = "UTF-8")
 
-      closeWS(.handle, code, reason)
-      .handle <<- NULL
-    }
+      closeWS(self$handle, code, reason)
+      self$handle <- NULL
+    },
+
+    handle = NULL,
+    messageCallbacks = list(),
+    closeCallbacks = list(),
+    request = NULL
   )
 )
 
 #' Create an HTTP/WebSocket server
-#' 
+#'
 #' Creates an HTTP/WebSocket server on the specified host and port.
-#' 
-#' @param host A string that is a valid IPv4 address that is owned by this 
+#'
+#' @param host A string that is a valid IPv4 address that is owned by this
 #'   server, or \code{"0.0.0.0"} to listen on all IP addresses.
 #' @param port A number or integer that indicates the server port that should be
 #'   listened on. Note that on most Unix-like systems including Linux and Mac OS
 #'   X, port numbers smaller than 1025 require root privileges.
-#' @param app A collection of functions that define your application. See 
+#' @param app A collection of functions that define your application. See
 #'   Details.
 #' @return A handle for this server that can be passed to
 #'   \code{\link{stopServer}} to shut the server down.
-#'   
+#'
 
 #' @details \code{startServer} binds the specified port and listens for
 #'   connections on an thread running in the background. This background thread
@@ -384,15 +427,23 @@ WebSocket <- setRefClass(
 #'   If the port cannot be bound (most likely due to permissions or because it
 #'   is already bound), an error is raised.
 #'
-#'   The \code{app} parameter is where your application logic will be provided 
-#'   to the server. This can be a list, environment, or reference class that 
-#'   contains the following named functions/methods:
-#'   
+#'   The application can also specify paths on the filesystem which will be
+#'   served from the background thread, without invoking \code{$call()} or
+#'   \code{$onHeaders()}. Files served this way will be only use a C++ code,
+#'   which is faster than going through R, and will not be blocked when R code
+#'   is executing. This can greatly improve performance when serving static
+#'   assets.
+#'
+#'   The \code{app} parameter is where your application logic will be provided
+#'   to the server. This can be a list, environment, or reference class that
+#'   contains the following methods and fields:
+#'
 #'   \describe{
-#'     \item{\code{call(req)}}{Process the given HTTP request, and return an 
+#'     \item{\code{call(req)}}{Process the given HTTP request, and return an
 #'     HTTP response. This method should be implemented in accordance with the
 #'     \href{https://github.com/jeffreyhorner/Rook/blob/a5e45f751/README.md}{Rook}
-#'     specification.}
+#'     specification.} Note that httpuv augments \code{req} with an additional item,
+#'     \code{req$HEADERS}, which is a named character vector of request headers.
 #'     \item{\code{onHeaders(req)}}{Optional. Similar to \code{call}, but occurs
 #'     when headers are received. Return \code{NULL} to continue normal
 #'     processing of the request, or a Rook response to send that response,
@@ -401,59 +452,95 @@ WebSocket <- setRefClass(
 #'     \item{\code{onWSOpen(ws)}}{Called back when a WebSocket connection is established.
 #'     The given object can be used to be notified when a message is received from
 #'     the client, to send messages to the client, etc. See \code{\link{WebSocket}}.}
+#'     \item{\code{staticPaths}}{
+#'       A named list of paths that will be served without invoking
+#'       \code{call()} or \code{onHeaders}. The name of each one is the URL
+#'       path, and the value is either a string referring to a local path, or an
+#'       object created by the \code{\link{staticPath}} function.
+#'     }
+#'     \item{\code{staticPathOptions}}{
+#'       A set of default options to use when serving static paths. If
+#'       not set or \code{NULL}, then it will use the result from calling
+#'       \code{\link{staticPathOptions}()} with no arguments.
+#'     }
 #'   }
-#'   
-#'   The \code{startPipeServer} variant can be used instead of 
+#'
+#'   The \code{startPipeServer} variant can be used instead of
 #'   \code{startServer} to listen on a Unix domain socket or named pipe rather
 #'   than a TCP socket (this is not common).
-#' @seealso \code{\link{stopServer}}, \code{\link{runServer}}
+#'
+#' @return A \code{\link{WebServer}} or \code{\link{PipeServer}} object.
+#'
+#' @seealso \code{\link{stopServer}}, \code{\link{runServer}},
+#'   \code{\link{listServers}}, \code{\link{stopAllServers}}.
 #' @aliases startPipeServer
+#'
+#' @examples
+#' \dontrun{
+#' # A very basic application
+#' s <- startServer("0.0.0.0", 5000,
+#'   list(
+#'     call = function(req) {
+#'       list(
+#'         status = 200L,
+#'         headers = list(
+#'           'Content-Type' = 'text/html'
+#'         ),
+#'         body = "Hello world!"
+#'       )
+#'     }
+#'   )
+#' )
+#'
+#' s$stop()
+#'
+#'
+#' # An application that serves static assets at the URL paths /assets and /lib
+#' s <- startServer("0.0.0.0", 5000,
+#'   list(
+#'     call = function(req) {
+#'       list(
+#'         status = 200L,
+#'         headers = list(
+#'           'Content-Type' = 'text/html'
+#'         ),
+#'         body = "Hello world!"
+#'       )
+#'     },
+#'     staticPaths = list(
+#'       "/assets" = "content/assets/",
+#'       "/lib" = staticPath(
+#'         "content/lib",
+#'         indexhtml = FALSE
+#'       ),
+#'       # This subdirectory of /lib should always be handled by the R code path
+#'       "/lib/dynamic" = excludeStaticPath()
+#'     ),
+#'     staticPathOptions = staticPathOptions(
+#'       indexhtml = TRUE
+#'     )
+#'   )
+#' )
+#'
+#' s$stop()
+#' }
 #' @export
 startServer <- function(host, port, app) {
-  
-  appWrapper <- AppWrapper$new(app)
-  server <- makeTcpServer(
-    host, port,
-    appWrapper$onHeaders,
-    appWrapper$onBodyData,
-    appWrapper$call,
-    appWrapper$onWSOpen,
-    appWrapper$onWSMessage,
-    appWrapper$onWSClose
-  )
-
-  if (is.null(server)) {
-    stop("Failed to create server")
-  }
-  return(server)
+  WebServer$new(host, port, app)
 }
 
-#' @param name A string that indicates the path for the domain socket (on 
+#' @param name A string that indicates the path for the domain socket (on
 #'   Unix-like systems) or the name of the named pipe (on Windows).
-#' @param mask If non-\code{NULL} and non-negative, this numeric value is used 
-#'   to temporarily modify the process's umask while the domain socket is being 
-#'   created. To ensure that only root can access the domain socket, use 
+#' @param mask If non-\code{NULL} and non-negative, this numeric value is used
+#'   to temporarily modify the process's umask while the domain socket is being
+#'   created. To ensure that only root can access the domain socket, use
 #'   \code{strtoi("777", 8)}; or to allow owner and group read/write access, use
 #'   \code{strtoi("117", 8)}. If the value is \code{NULL} then the process's
 #'   umask is left unchanged. (This parameter has no effect on Windows.)
 #' @rdname startServer
 #' @export
 startPipeServer <- function(name, mask, app) {
-  
-  appWrapper <- AppWrapper$new(app)
-  if (is.null(mask))
-    mask <- -1
-  server <- makePipeServer(name, mask,
-                           appWrapper$onHeaders,
-                           appWrapper$onBodyData,
-                           appWrapper$call,
-                           appWrapper$onWSOpen,
-                           appWrapper$onWSMessage,
-                           appWrapper$onWSClose)
-  if (is.null(server)) {
-    stop("Failed to create server")
-  }
-  return(server)
+  PipeServer$new(name, mask, app)
 }
 
 #' Process requests
@@ -488,9 +575,14 @@ startPipeServer <- function(name, mask, app) {
 #' @export
 #' @importFrom later run_now
 service <- function(timeoutMs = ifelse(interactive(), 100, 1000)) {
+  # In all cases, call `run_now` with `all = FALSE` so that if there is a lot of
+  # incoming traffic (relative to the time it takes to process it) we give the
+  # owning event loop opportunities to do housekeeping in between httpuv related
+  # callbacks.
+
   if (is.na(timeoutMs)) {
     # NA means to run non-blocking
-    run_now(0)
+    run_now(0, all = FALSE)
 
   } else if (timeoutMs == 0 || timeoutMs == Inf) {
     .globals$paused <- FALSE
@@ -498,13 +590,13 @@ service <- function(timeoutMs = ifelse(interactive(), 100, 1000)) {
     # responsiveness when the user sends an interrupt (like Esc in RStudio.)
     check_time <- if (interactive()) 0.1 else Inf
     while (!.globals$paused) {
-      run_now(check_time)
+      run_now(check_time, all = FALSE)
     }
 
   } else {
     # No need to check for .globals$paused because if run_now() executes
     # anything, it will return immediately.
-    run_now(timeoutMs / 1000)
+    run_now(timeoutMs / 1000, all = FALSE)
   }
 
   # Some code expects service() to return TRUE (#123)
@@ -512,31 +604,49 @@ service <- function(timeoutMs = ifelse(interactive(), 100, 1000)) {
 }
 
 #' Run a server
-#' 
-#' This is a convenience function that provides a simple way to call 
-#' \code{\link{startServer}}, \code{\link{service}}, and 
-#' \code{\link{stopServer}} in the correct sequence. It does not return unless 
+#'
+#' This is a convenience function that provides a simple way to call
+#' \code{\link{startServer}}, \code{\link{service}}, and
+#' \code{\link{stopServer}} in the correct sequence. It does not return unless
 #' interrupted or an error occurs.
-#' 
-#' If you have multiple hosts and/or ports to listen on, call the individual 
+#'
+#' If you have multiple hosts and/or ports to listen on, call the individual
 #' functions instead of \code{runServer}.
-#' 
-#' @param host A string that is a valid IPv4 address that is owned by this 
+#'
+#' @param host A string that is a valid IPv4 address that is owned by this
 #'   server, or \code{"0.0.0.0"} to listen on all IP addresses.
 #' @param port A number or integer that indicates the server port that should be
 #'   listened on. Note that on most Unix-like systems including Linux and Mac OS
 #'   X, port numbers smaller than 1025 require root privileges.
-#' @param app A collection of functions that define your application. See 
+#' @param app A collection of functions that define your application. See
 #'   \code{\link{startServer}}.
 #' @param interruptIntervalMs Deprecated (last used in httpuv 1.3.5).
-#'   
+#'
 #' @seealso \code{\link{startServer}}, \code{\link{service}},
 #'   \code{\link{stopServer}}
+#'
+#' @examples
+#' \dontrun{
+#' # A very basic application
+#' runServer("0.0.0.0", 5000,
+#'   list(
+#'     call = function(req) {
+#'       list(
+#'         status = 200L,
+#'         headers = list(
+#'           'Content-Type' = 'text/html'
+#'         ),
+#'         body = "Hello world!"
+#'       )
+#'     }
+#'   )
+#' )
+#' }
 #' @export
 runServer <- function(host, port, app, interruptIntervalMs = NULL) {
   server <- startServer(host, port, app)
   on.exit(stopServer(server))
-  
+
   # TODO: in the future, add deprecation message to interruptIntervalMs.
   service(0)
 }
@@ -555,17 +665,17 @@ interrupt <- function() {
 }
 
 #' Convert raw vector to Base64-encoded string
-#' 
+#'
 #' Converts a raw vector to its Base64 encoding as a single-element character
 #' vector.
-#' 
+#'
 #' @param x A raw vector.
-#'   
+#'
 #' @examples
 #' set.seed(100)
 #' result <- rawToBase64(as.raw(runif(19, min=0, max=256)))
 #' stopifnot(identical(result, "TkGNDnd7z16LK5/hR2bDqzRbXA=="))
-#' 
+#'
 #' @export
 rawToBase64 <- function(x) {
   base64encode(x)
@@ -586,16 +696,6 @@ rawToBase64 <- function(x) {
 #' @export
 startDaemonizedServer <- startServer
 
-#' Stop a running daemonized server in Unix environments (deprecated)
-#'
-#' This function will be removed in a future release of httpuv. Instead, use
-#' \code{\link{stopServer}}.
-#'
-#' @inheritParams stopServer
-#'
-#' @export
-stopDaemonizedServer <- stopServer
-
 
 # Needed so that Rcpp registers the 'httpuv_decodeURIComponent' symbol
 legacy_dummy <- function(value){
